# -*- coding: utf-8 -*-
require 'isono'

module Dcmgr::NodeModules

  class AlarmConfigUpdater < Isono::NodeModules::Base
    include Isono::NodeModules
    include Dcmgr::Logger
    include Dcmgr::Helpers::TemplateHelper

    initialize_hook do
        app = Isono::Rack::ObjectMethod.new(myinstance)
        job = Isono::NodeModules::JobChannel.new(node)
        job.register_endpoint("log-alarm-registry.#{self.node.node_id}", Isono::Rack.build do
                                run proc { |req, res|
                                  app.call(req, res)
                                }
                              end)
    end

    terminate_hook do
    end

    def update_alarm
      alarms = get_alarms
      write_config(alarms)
    end

    def delete_alarm
      alarms = get_alarms
      write_config(alarms)
    end

    def delete_alarms
      alarms = get_alarms
      write_config(alarms)
    end

    private

    def get_alarms
      rpc.request('alarm-collector', 'get_log_alarms', self.node.node_id)
    end

    def write_config(alarms)
      alm = []
      alarms.each do |a|

        h = {
          :resource_id => a[:resource_id],
          :alarm_id => a[:uuid],
          :tag => a[:params]['tag'],
          :match_pattern => a[:params]['match_pattern'],
          :notification_periods => a[:notification_periods],
          :enabled => a[:enabled],
          :alarm_action => '',
          :display_name => a[:display_name]
        }

        if a[:alarm_actions]
          h[:alarm_action] = format_notification_action(
                                a[:alarm_actions]['notification_id'],
                                a[:alarm_actions]['notification_message_type'])
        end

        alm << h

      end

      fluent = Dcmgr::Drivers::Fluent.new
      fluent.set_alarms(alm)
      fluent.generate_config
      fluent.reload
    end

    def format_notification_action(id, message_type)
      "#{id}:#{message_type}"
    end

    def rpc
      @rpc ||= Isono::NodeModules::RpcChannel.new(@node)
    end

  end
end
