# -*- coding: utf-8 -*-
module Dcmgr::NodeModules
  class Maintenance < Isono::NodeModules::Base
    include Dcmgr::Logger

    initialize_hook do
      @thread_pool = Isono::ThreadPool.new(1, 'Maintenance')
      @thread_pool.pass {
        myinstance.dup.expire_ip_handles
        myinstance.dup.automatic_windows_password_deletion
      }

      @maintenace = EventMachine::PeriodicTimer.new(60) {
        next if @thread_pool.queue.size > 0
        @thread_pool.pass {
          myinstance.dup.expire_ip_handles
          myinstance.dup.automatic_windows_password_deletion
        }
      }
    end

    terminate_hook do
      @maintenace.cancel
      @thread_pool.shutdown
    end

    def expire_ip_handles
      # logger.debug "Checking for expired IP handles."

      Dcmgr::Models::IpHandle.dataset.not_leased.expired.alives.each { |handle|
        logger.debug "IP handle expired: uuid:#{handle.canonical_uuid} network:#{handle.ip_lease.network.canonical_uuid} ipv4:#{handle.ip_lease.ipv4_s}"

        handle.destroy
      }
    end

    def automatic_windows_password_deletion
      Dcmgr::Models::Instance.dataset.alives.password_deletion_time_passed.each { |inst|
        logger.debug "Automatically deleting encrypted password for instance with uuid: %s" %
                     inst.canonical_uuid
        inst.delete_windows_password
      }
    end

  end

end
